@@ -28,7 +28,6 @@
 GtkWidget* w_about[W_ABOUT_N];
 
 void about_create(void) {
-  GtkAboutDialog* about;
   GdkPixbuf* logo;
   
   char* authors[] = { "Claudio Fontana <claudio@gnu.org>", 0 };
@@ -58,8 +57,15 @@ void about_create(void) {
   char* version = PACKAGE_VERSION;
   char* website = "http://www.gnu.org/software/sourceinstall/";
 
+#if GTK_CHECK_VERSION(2, 6, 0)
+  GtkAboutDialog* about;
+#else
+  GtkWidget* about;
+#endif
+
   logo = gdk_pixbuf_new_from_file(DATADIR "/images/logo.png", NULL);
 
+#if GTK_CHECK_VERSION(2, 6, 0)
   about = GTK_ABOUT_DIALOG(gtk_about_dialog_new());
   gtk_about_dialog_set_name(about, name);
   gtk_about_dialog_set_version(about, version);
@@ -72,6 +78,12 @@ void about_create(void) {
 
   if (logo)
     gtk_about_dialog_set_logo(about, logo);
+#else
+  about = dialog_create_msgbox_scroll("About GNU Source Installer",
+				      PACKAGE, comments, license, copyright,
+				      GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
+#endif
+
   
   w_about[W_ABOUT] = GTK_WIDGET(about);
 }
