@@ -38,15 +38,6 @@ static int action_find_configure(SRCINST_PHANDLE h, char* filename);
 static int action_configure(SRCINST_PHANDLE h);
 static void action_start(void);
 static void action_end(void);
-static void action_error(SRCINST_ERR rv, char* title, char* text, char* when);
-
-static void action_error(SRCINST_ERR rv, char* title, char* text, char* when) {
-  if (rv == SRCINST_ERR_INTR) {
-    dialog_warning(title, "Action interrupted.", "An interrupt signal has been received while", when);
-  } else {
-    dialog_warning(title, text, "This error condition has been detected while", when);
-  }
-}
 
 void action_add(char* filename, char* packagename) {
   SRCINST_PHANDLE h; int result;
@@ -65,7 +56,7 @@ void action_add(char* filename, char* packagename) {
   
   if (!(h = srcinst_new_package(packagename))) {
     dialog_warning(title, "Package already exists",
-		   "Could not create already existing package", packagename);
+		   "Could not create already existing package:", packagename);
     goto end_proc;
   }
 
@@ -76,7 +67,8 @@ void action_add(char* filename, char* packagename) {
   
  end_proc:
   if (h && !srcinst_close_package(h)) {
-    dialog_warning(title, "Error updating package information", "An error condition has been detected trying to update and close the package information.", 0);
+    dialog_warning(title, "Error updating package information",
+		   "Could not update and close the package information.", 0);
     result = 0;
   }
 
@@ -84,7 +76,8 @@ void action_add(char* filename, char* packagename) {
   action_end();
 
   if (result) {
-    dialog_message(title, "Package added successfully.", "Your new package has been successfully added.", 0);
+    dialog_message(title, "Package added successfully", 
+		   "Your new package has been successfully added.", 0);
   }
 }
 
@@ -98,7 +91,8 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
   status_done(0.0, 10.0);
 
   if ((rv = srcinst_init_build(h, filename, 0)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Could not initialize build", "initializing the build prior to installation.");
+    dialog_warning(title, srcinst_error_string(rv), 
+		   "Could not initialize build.", 0);
     return 0;
   }
 
@@ -106,7 +100,8 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
 
   if (reconf) {
     if ((rv = srcinst_chdir_build(h, 0)) != SRCINST_ERR_OK) {
-      action_error(rv, title, "Could not change build directory", "preparing to reconfigure package.");
+      dialog_warning(title, "Could not change build directory", 
+	     "Could not change build directory to reconfigure package.", 0);
       return 0;
     }
 
@@ -117,7 +112,8 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
 	return 0;
       
     } else if (rv != SRCINST_ERR_OK) {
-      action_error(rv, title, "Error detecting configuration script", "reconfiguring package prior to installation.");
+      dialog_warning(title, srcinst_error_string(rv),
+		     "Error detecting configuration script.", 0);
       return 0;
     }
 
@@ -130,21 +126,38 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
   
   status_done(20.0, 25.0);
 
-  if (!(mf = srcinst_detect_makefile_features(h))) {
-    dialog_warning(title, "Makefile detection failed.", "Failed to detect Makefile or Makefile features.", 0);
+  srcinst_unregister_stdout_callback(console_print_stdout);
+  mf = srcinst_detect_makefile_features(h);
+  srcinst_register_stdout_callback(console_print_stdout);
+
+  if (!mf) {
+    dialog_warning(title, "Makefile detection failed", 
+		   "Failed to detect Makefile or Makefile features.", 0);
     return 0;
   }
   
   if (!srcinst_is_feature_supported(mf, SRCINST_FEATURE_DESTDIR) &&
       !srcinst_is_feature_supported(mf, SRCINST_FEATURE_INSTALL_ROOT)) {
-    dialog_warning(title, "Staged installation not supported", "This package does not support required staged installation features.", 0);
-    srcinst_free_makefile_features(mf);
-    return 0;
+
+    if (!srcinst_is_feature_supported(mf, SRCINST_FEATURE_PREFIX)) {
+      dialog_warning(title, "Staged installation not supported",
+		     "This package does not support staged installation.",
+		     0);
+      srcinst_free_makefile_features(mf);
+      return 0;
+      
+    } else if (!dialog_confirm(title, "Staged installation not supported",
+			       "This package does not support staged installation.\nHowever, prefix redirection could work. It is error-prone and discouraged.\n\nDo you want to try using prefix redirection?", 0, 0)) {
+      srcinst_free_makefile_features(mf);
+      return 0;
+    }
   }
 
   if (state.o.source) {
     if ((rv = srcinst_install_source(h, mf, state.o.src_compress)) != SRCINST_ERR_OK) {
-      action_error(rv, title, "Source installation failed", "installing the configured source code for later use. Check console for the reasons behind the error.");
+      dialog_warning(title, srcinst_error_string(rv),
+		     "Could not install the configured source code.\n"
+		     "Check console for the reasons behind the error.", 0);
       srcinst_free_makefile_features(mf);
       return 0;
     }
@@ -153,7 +166,11 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
   status_done(25.0, 90.0);
   
   if ((rv = srcinst_build(h, mf)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Build failed", "compiling the source code. Check if your system is supported by the package you want to install, and check configuration options. It could also be a bug in the package itself. You can generate a bug report quickly by right-clicking in the console window, and choosing Save Log.");
+    dialog_warning(title, srcinst_error_string(rv), 
+		   "Package build processes failed.\n"
+		   "Remember that you can get information for a bug report\n"
+		   "quickly by right-clicking in the console window, and \n"
+		   "choosing Save Log.", 0);
     srcinst_free_makefile_features(mf); return 0;
   }
   
@@ -161,7 +178,9 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
     status_done(90.0, 92.5);
 
     if ((rv = srcinst_test_install(h, mf, state.o.strip)) != SRCINST_ERR_OK) {
-      action_error(rv, "Test installation failed", "performing the test installation. Therefore, real installation will not be attempted.", 0);
+      dialog_warning(title, srcinst_error_string(rv), 
+		     "Test installation failed.\n"
+		     "Real installation will not be attempted.", 0);
       srcinst_free_makefile_features(mf); return 0;
     }
 
@@ -169,7 +188,8 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
       struct srcinst_string_list conflicts;
       
       if (!srcinst_get_conflict_list(h, &conflicts)) {
-	dialog_warning(title, "Conflict test failed", "An unexpected error has occured trying to perform the package conflict test", 0);
+	dialog_warning(title, "Conflict test failed", 
+		       "Could not perform the package conflict test.", 0);
 	srcinst_free_makefile_features(mf); return 0;
       }
 
@@ -182,10 +202,9 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
 					    conflicts.strings[i]);
 	}
 
-	conflicts_string = srcinst_append(conflicts_string, "\n\n  Do you really want to overwrite the already claimed files?");
-	i = dialog_confirm(title, "Conflicts detected", "Installing this package would overwrite files which other packages claim. The files involved are:\n", conflicts_string);
+	i = dialog_confirm(title, "Conflicts detected", "Installing this package would overwrite files of other packages!\nThe files involved are:\n", conflicts_string, "Do you really want to overwrite the already claimed files?");
 	free(conflicts_string);
-
+	
 	if (!i) {
 	  srcinst_free_string_list(&conflicts);
 	  srcinst_free_makefile_features(mf);
@@ -198,7 +217,9 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int
     status_done(92.5, 100.0);
     
     if ((rv = srcinst_install_binary(h, mf, state.o.strip)) != SRCINST_ERR_OK) {
-      action_error(rv, title, "Binary installation failed", "installing binaries in their final location. See the console for the reasons behind the error. Remember to check your permissions on the chosen installation prefix.");
+      dialog_warning(title, srcinst_error_string(rv), 
+		     "Failed to install binaries to their final location.\n"
+		     "See the console for the reasons behind the error.", 0);
       srcinst_free_makefile_features(mf); return 0;
     }
   }
@@ -221,7 +242,8 @@ static int action_configure(SRCINST_PHANDLE h) {
 
   if (state.o.manual_conf) {
     if ((rv = srcinst_detect_configure_opt(h, &sets)) != SRCINST_ERR_OK) {
-      action_error(rv, title, "Failed to detect configure options", "trying to detect supported configure options.");
+      dialog_warning(title, srcinst_error_string(rv),
+		     "Failed to detect configure options.", 0);
       if (rv == SRCINST_ERR_INTR)
 	goto end_proc;
 
@@ -236,7 +258,9 @@ static int action_configure(SRCINST_PHANDLE h) {
   
   if ((rv = srcinst_configure(h, state.o.prefix, &configure_strings)) !=
       SRCINST_ERR_OK) {
-    action_error(rv, title, "Configuration failed!", "running configuration scripts.");
+    dialog_warning(title, srcinst_error_string(rv), 
+		   "Configuration script failed.\n"
+		   "Check the console for the reasons behind the error.\n", 0);
     goto end_proc;
   }
 
@@ -250,19 +274,21 @@ static int action_configure(SRCINST_PHANDLE h) {
 static int action_find_configure(SRCINST_PHANDLE h, char* filename) {
   struct srcinst_string_list scripts;
   SRCINST_ERR rv;
-  int result;
+  int result = 0;
   char* subdir = 0;
   char* title = "Find configure script";
       
   srcinst_init_string_list(&scripts);
 
   if ((rv = srcinst_find_configure(h, &scripts)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Error searching for configuration script", "searching for a configuration script.");
+    dialog_warning(title, srcinst_error_string(rv), 
+		   "Error searching for a suitable configuration script.", 0);
     goto end_proc;
   }
 
   if (scripts.count == 0) {
-    action_error(rv, title, "No configuration script available", "trying to detect a suitable configuration script, or script-generation script");
+    dialog_warning(title, "No configuration script available",
+   "Could not detect a suitable script or script-generating script", 0);
     goto end_proc;
   }
 
@@ -279,7 +305,8 @@ static int action_find_configure(SRCINST_PHANDLE h, char* filename) {
   }
 
   if ((rv = srcinst_chdir_build(h, subdir)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Could not change build subdirectory", "attempting to move to the chosen build subdirectory.");
+    dialog_warning(title, srcinst_error_string(rv), 
+		   "Could not move to the build subdirectory.", 0);
     goto end_proc;
   }
 
@@ -303,7 +330,9 @@ void action_install(int reconf) {
   result = 0;
   
   if (!(source = srcinst_get_source_location(state.package))) {
-    dialog_warning(title, "Failed to find source code", "This package source code is not reachable, even though it should be available. Installation impossible.", 0);
+    dialog_warning(title, "Failed to find source code", 
+		   "This package source code is not reachable, \n"
+		   "even though it should be available.", 0);
     goto end_proc;
   }
   
@@ -314,7 +343,8 @@ void action_install(int reconf) {
 
  end_proc:
   if (!srcinst_close_package(state.package)) {
-    dialog_warning(title, "Error closing package", "An unexpected error has occured when finalizing and closing the package.", 0);
+    dialog_warning(title, "Error closing package",
+		   "Error trying to finalize and close the package.", 0);
     result = 0;
   }
 
@@ -342,7 +372,9 @@ void action_remove(void) {
 			      srcinst_get_source_location(state.package), 0);
       
       if (rv != SRCINST_ERR_OK) {
-	action_error(rv, title, "Could not initialize build", "initializing source build for source-assisted uninstallation.");
+	dialog_warning(title, srcinst_error_string(rv),
+		       "Could not initialize build for source-assisted "
+		       "uninstallation.", 0);
 	goto end_proc;
       }
     }
@@ -350,7 +382,8 @@ void action_remove(void) {
     status_done(10.0, 95.0);
 
     if ((rv = srcinst_uninstall_binary(state.package)) != SRCINST_ERR_OK) {
-      action_error(rv, title, "Could not uninstall binaries", "deleting the installed files.");
+      dialog_warning(title, srcinst_error_string(rv), 
+		     "Could not uninstall binaries.", 0);
       goto end_proc;
     }
   }
@@ -359,7 +392,8 @@ void action_remove(void) {
     
   if (srcinst_is_source_available(state.package)) {
     if ((rv = srcinst_uninstall_source(state.package)) != SRCINST_ERR_OK) {
-      action_error(rv, title, "Could not remove sources", "trying to delete the stored source code");
+      dialog_warning(title, srcinst_error_string(rv), 
+		     "Could not remove stored source code", 0);
       goto end_proc;
     }
   }
@@ -369,7 +403,8 @@ void action_remove(void) {
 
  end_proc:
   if (!srcinst_close_package(state.package)) {
-    dialog_warning(title, "Error closing package", "An unexpected error has occured when finalizing and closing the package.", 0);
+    dialog_warning(title, "Error closing package", 
+		   "Could not finalize and close the package.", 0);
     result = 0;
   }
 
@@ -396,7 +431,9 @@ void action_uninstall(void) {
 			    srcinst_get_source_location(state.package), 0);
 
     if (rv != SRCINST_ERR_OK) {
-      action_error(rv, title, "Could not initialize build", "initializing source build for source-assisted uninstallation.");
+      dialog_warning(title, srcinst_error_string(rv), 
+		     "Could not initialize build "
+		     "for source-assisted uninstallation.", 0);
       
       goto end_proc;
     }
@@ -405,7 +442,8 @@ void action_uninstall(void) {
   status_done(10.0, 98.0);
 
   if ((rv = srcinst_uninstall_binary(state.package)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Could not uninstall binaries", "deleting the installed files.");
+    dialog_warning(title, srcinst_error_string(rv), 
+		   "Could not uninstall binaries.", 0);
     goto end_proc;
   }
 
@@ -415,7 +453,8 @@ void action_uninstall(void) {
  end_proc:
 
   if (!srcinst_close_package(state.package)) {
-    dialog_warning(title, "Error closing package", "An unexpected error has occured when finalizing and closing the package.", 0);
+    dialog_warning(title, "Error closing package",
+		   "Could not finalize and close the package.", 0);
     result = 0;
   }
 
@@ -423,18 +462,18 @@ void action_uninstall(void) {
   action_end();
 
   if (result) {
-    dialog_message(title, "Package uninstalled.", "The package has been uninstalled successfully.", 0);
+    dialog_message(title, "Package uninstalled", "The package has been uninstalled successfully.", 0);
   }
 }
 
 void action_upgrade(char* filename, char* packagename) {
-  SRCINST_PHANDLE h; int result;
+  SRCINST_PHANDLE h; int rv;
   char* title = "Upgrade package";
-  char* descr; char* descr_l;
+  char* oldname;
 
   SRCINST_TYPE ft;
 
-  result = 0; h = 0;
+  rv = 0; h = 0;
   action_start();
 
   ft = srcinst_file_type(filename);
@@ -444,39 +483,43 @@ void action_upgrade(char* filename, char* packagename) {
     goto end_proc;
   }
 
-  if (!(h = srcinst_new_package(packagename))) {
-    dialog_warning(title, "Package already exists",
-		   "Could not create already existing package", packagename);
+  if (!(oldname = srcinst_get_name(state.package))) {
+    dialog_warning(title, "Invalid old package",
+		   "Could not read old package name.", 0);
+    goto end_proc;
+  }
+  
+  /* XXX old package not closed!
+     srcinst_close_package(state.package); */
+  
+  if (!(h = srcinst_upgrade_package(packagename, oldname))) {
+    dialog_warning(title, "Cannot start package upgrade",
+		   "Could not start the upgrade procedure, "
+		   "check that the old package is installed, "
+		   "and that the new package name is valid", 0);
     goto end_proc;
   }
 
   if (!action_install_aux(h, filename, 1, 0))
     goto end_proc;
 
-  if ((descr = srcinst_get_descr(state.package))) {
-    srcinst_set_descr(h, descr);
-  }
-  
-  if ((descr_l = srcinst_get_descr_long(state.package))) {
-    srcinst_set_descr_long(h, descr_l);
-  }
-
-  result = 1;
+  rv = 1;
   
  end_proc:
-
-  if (result) {
-    SRCINST_ERR rv;
-    if ((rv = srcinst_upgrade_package(state.package, h)) != SRCINST_ERR_OK) {
-      action_error(rv, title, "Error upgrading package information.", "trying to upgrade package information.");
-    }
+  
+  if (h) {
+    rv = srcinst_close_package(h) && rv;
   }
   
   status_done(100.0, 100.0);
   action_end();
 
-  if (result) {
-    dialog_message(title, "Package upgraded successfully.", "Your package has been successfully upgraded.", 0);
+  if (rv) {
+    dialog_message(title, "Package upgraded successfully.",
+		   "Your package has been successfully upgraded.", 0);
+  } else {
+    dialog_message(title, "Package _not_ upgraded.",
+		   "The old package has been restored.", 0);
   }
 }
 
@@ -485,7 +528,8 @@ void action_export_info(SRCINST_PHANDLE h, char* fname, SRCINST_EXPORT fmt) {
   char* title = "Export information";
 
   if ((rv = srcinst_export_info(h, fname, fmt)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Failed to export information", "trying to export package information to the chosen file.");
+    dialog_warning(title, srcinst_error_string(rv),
+		   "Failed to export the package information.", 0);
 
   } else {
     dialog_message(title, "Information successfully exported", "Package information has been successfully written to", fname);
@@ -501,7 +545,8 @@ void action_export_bin(char* fname, SRCINST_COMP fmt) {
   status_done(0.0, 100.0);
 
   if ((rv = srcinst_export_binary(h, fname, fmt)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Failed to export binary package", "trying to write the chosen file.");
+    dialog_warning(title, srcinst_error_string(rv), 
+		   "Failed to export binary package.", 0);
 
   }
 
