@@ -25,23 +25,147 @@
 #include "gui.h"
 #include "dialog.h"
 
+/* this function is called by many other dialogs to display a message
+   box with buttons, and possibly a scroll for text2 */
+
+extern GtkWidget*
+dialog_create_msgbox_scroll(char* wintitle, char* title, 
+			    char* text1, char* text2, char* text3,
+			    GtkMessageType type, GtkButtonsType buttons);
+
+static GtkWidget*
+dialog_create_msgbox(char* wintitle, char* title,
+		     char* text, char* opt, 
+		     GtkMessageType type, GtkButtonsType buttons) {
+
+  return dialog_create_msgbox_scroll(wintitle, title, text, 0, opt,
+				     type, buttons);
+}
+
+extern GtkWidget*
+dialog_create_msgbox_scroll(char* wintitle, char* title,
+			    char* text1, char* text2, char* text3,
+			    GtkMessageType type, GtkButtonsType buttons) {
+  GtkWidget* msgbox;
+
+  switch (buttons) {
+
+  default:
+  case GTK_BUTTONS_NONE:
+  case GTK_BUTTONS_OK:
+    msgbox = gtk_dialog_new_with_buttons(wintitle, GTK_WINDOW(w_main[W_MAIN]),
+					 GTK_DIALOG_MODAL,
+					 GTK_STOCK_OK, GTK_RESPONSE_OK,
+					 0);
+    break;
+  case GTK_BUTTONS_CLOSE:
+    msgbox = gtk_dialog_new_with_buttons(wintitle, GTK_WINDOW(w_main[W_MAIN]),
+					 GTK_DIALOG_MODAL,
+					 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					 0);
+    break;
+  case GTK_BUTTONS_CANCEL:
+    msgbox = gtk_dialog_new_with_buttons(wintitle, GTK_WINDOW(w_main[W_MAIN]),
+					 GTK_DIALOG_MODAL,
+					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					 0);
+    break;
+  case GTK_BUTTONS_YES_NO:
+    msgbox = gtk_dialog_new_with_buttons(wintitle, GTK_WINDOW(w_main[W_MAIN]),
+					 GTK_DIALOG_MODAL,
+					 GTK_STOCK_YES, GTK_RESPONSE_YES,
+					 GTK_STOCK_NO, GTK_RESPONSE_NO,
+					 0);
+    break;
+  case GTK_BUTTONS_OK_CANCEL:
+    msgbox = gtk_dialog_new_with_buttons(wintitle, GTK_WINDOW(w_main[W_MAIN]),
+					 GTK_DIALOG_MODAL,
+					 GTK_STOCK_OK, GTK_RESPONSE_OK,
+					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					 0);
+    break;
+  }
+
+  {
+    GtkWidget* vbox, *hbox, *title_txt, *img;
+    
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(msgbox)->vbox), hbox, 
+		       TRUE, TRUE, 0);
+
+    img = gtk_image_new_from_stock(type == GTK_MESSAGE_ERROR ?
+				   GTK_STOCK_DIALOG_ERROR :
+				   type == GTK_MESSAGE_WARNING ?
+				   GTK_STOCK_DIALOG_WARNING :
+				   type == GTK_MESSAGE_QUESTION ?
+				   GTK_STOCK_DIALOG_QUESTION :
+				   GTK_STOCK_DIALOG_INFO, 
+
+				   GTK_ICON_SIZE_DIALOG);
+
+    gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, bw);
+				   
+    vbox = gtk_vbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
+    
+    title_txt = gtk_label_new(NULL);
+    gtk_box_pack_start(GTK_BOX(vbox), title_txt, FALSE, FALSE, bw);
+
+    title = srcinst_strjoin("<b>", title, "</b>", 0);
+    gtk_label_set_markup(GTK_LABEL(title_txt), title);
+    free(title);
+
+    if (text1) {
+      GtkWidget *txt1;
+      txt1 = gtk_label_new(text1);
+      gtk_box_pack_start(GTK_BOX(vbox), txt1, FALSE, FALSE, bw);
+    }
+    
+    if (text2) {
+      GtkWidget* scroll, *txt2;
+
+      scroll = gtk_scrolled_window_new(NULL, NULL);
+      txt2 = gtk_label_new(text2);
+
+      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
+				     GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll), txt2);
+
+      gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
+    }
+
+    if (text3) {
+      GtkWidget *txt3;
+
+      txt3 = gtk_label_new(text3);
+      gtk_box_pack_start(GTK_BOX(vbox), txt3, FALSE, FALSE, bw);
+    }
+    
+    {
+      GtkWidget *space;
+      space = gtk_label_new(NULL);
+      gtk_box_pack_start(GTK_BOX(hbox), space, FALSE, FALSE, bw / 2);
+    }
+  }
+
+  { 
+    GdkGeometry geo;
+    geo.max_aspect = 8.0 / 4.0;
+    geo.min_aspect = 3.0 / 4.0;
+
+    gtk_window_set_geometry_hints(GTK_WINDOW(msgbox),
+				  0, &geo, GDK_HINT_ASPECT);
+  }
+
+  gtk_widget_show_all(GTK_DIALOG(msgbox)->vbox);
+  return msgbox;
+}
+
+
 /* display an error dialog, with window title WINTITLE,
    dialog title TITLE, additional TEXT and an optional 
    additional text argument OPT. Use NULL to avoid. Then quit. */
 
-static GtkWidget* dialog_create_msgbox(char* wintitle, char* title, char* text, char* opt, GtkMessageType type, GtkButtonsType buttons) {
-  GtkWidget* msgbox;
-
-  msgbox = gtk_message_dialog_new(GTK_WINDOW(w_main[W_MAIN]), GTK_DIALOG_MODAL,
-				  type, buttons, "%s", title);
-  
-  gtk_window_set_title(GTK_WINDOW(msgbox), wintitle);
-  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(msgbox),
-				  "%s %s", text ? text : "", opt ? opt : "");
-
-  return msgbox;
-}
-
 void dialog_error(char* wintitle, char* title, char* text, char* opt) {
   GtkWidget* msgbox;
 
@@ -64,14 +188,18 @@ void dialog_warning(char* wintitle, char* title, char* text, char* opt) {
 }
 
 /* Asks the user for confermation. This function implies a YES|NO response,
-   which is returned. */
+   which is returned. Text2 and Text3 are optional. */
 
-int dialog_confirm(char* wintitle, char* title, char* text, char* opt) {
+int dialog_confirm(char* wintitle, char* title, 
+		   char* text1, char* text2, char* text3) {
   GtkWidget* msgbox;
   gint response;
 
-  msgbox = dialog_create_msgbox(wintitle, title, text, opt,
-				GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO);
+  msgbox = dialog_create_msgbox_scroll(wintitle, title,
+				       text1, text2, text3,
+				       GTK_MESSAGE_QUESTION,
+				       GTK_BUTTONS_YES_NO);
+
   response = gtk_dialog_run(GTK_DIALOG(msgbox));
   gtk_widget_destroy(msgbox);
   return response == GTK_RESPONSE_YES;
@@ -82,8 +210,9 @@ int dialog_confirm(char* wintitle, char* title, char* text, char* opt) {
 void dialog_message(char* wintitle, char* title, char* text, char* opt) {
   GtkWidget* msgbox;
 
-  msgbox = dialog_create_msgbox(wintitle, title, text, opt,
-				GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
+  msgbox = dialog_create_msgbox_scroll(wintitle, title, text, opt, 0,
+				       GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
+
   gtk_dialog_run(GTK_DIALOG(msgbox));
   gtk_widget_destroy(msgbox);
 }
@@ -494,7 +623,8 @@ int dialog_configure(struct srcinst_configure_sets* opts, struct srcinst_string_
 			     param ? "]\n" : "\n", comment ? comment : "", 0);
 
       gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), columns[i].check_buttons[j],
-			   text, 0);
+                           text, 0);
+
       free(text);
     }
   }
