@@ -164,9 +164,9 @@ void gui_maybe_quit(void) {
 
   if (state.actions_locked) {
     quit = dialog_confirm(title, "Action in progress!",
-		       "A quit request has been detected, but currently "
-		       "executing action is not complete.\n\n"
-		       "Are you sure that you want to quit?", 0);
+			  "A quit request has been detected,\n"
+			  "but current action is not complete.\n", 0,
+			  "Are you sure that you want to quit?");
   }
   
   if (quit) {
@@ -212,7 +212,7 @@ void gui_action_add(GtkWidget* w, gpointer data) {
 void gui_action_remove(GtkWidget* w, gpointer data) {
   printf("gui_action_remove\n");
 
-  if (dialog_confirm("Remove package", "Completely remove package?", srcinst_get_name(state.package), "will be uninstalled, and its stored source removed. Really proceed?"))
+  if (dialog_confirm("Remove package", "Completely remove package?", "Following package will be uninstalled, and its stored source removed:", srcinst_get_name(state.package), "Really proceed?"))
     action_remove();
 }
 
@@ -221,7 +221,7 @@ void gui_action_install(GtkWidget* w, gpointer data) {
   printf("gui_action_install\n");
 
   if (srcinst_get_configure_string(state.package)) {
-    if (!dialog_confirm("Install package", "Reconfigure package?", "This package is already configured, and you might want to keep the same configuration. Do you want to reconfigure the package?", 0))
+    if (!dialog_confirm("Install package", "Reconfigure package?", "This package is already configured, and you might want to keep the same configuration.", 0, "Do you want to reconfigure the package?"))
       reconf = 0;
   }
 
@@ -231,7 +231,7 @@ void gui_action_install(GtkWidget* w, gpointer data) {
 void gui_action_uninstall(GtkWidget* w, gpointer data) {
   printf("gui_action_uninstall\n");
 
-  if (dialog_confirm("Uninstall package", "Uninstall package?", srcinst_get_name(state.package), "installed files will be deleted; really proceed?"))
+  if (dialog_confirm("Uninstall package", "Uninstall package?", "Following package will be uninstalled:", srcinst_get_name(state.package), "Really proceed?"))
     action_uninstall();
 }
 
@@ -254,8 +254,8 @@ void gui_action_upgrade(GtkWidget* w, gpointer data) {
 			    &filename, &packagename)) {
     /* printf("filename: %s\npackname: %s\n", filename, packagename); */
     char* txt; int c;
-    txt = srcinst_strjoin(filename, " will be built, and if successful, current package ", srcinst_get_name(state.package), " will be replaced by the new version. Proceed?", 0);
-    c = dialog_confirm("Upgrade package", "Upgrade current package?", txt, 0);
+    txt = srcinst_strjoin(filename, "\nwill be built, and if successful, current package\n\"", srcinst_get_name(state.package), "\" will be replaced with the new version.", 0);
+    c = dialog_confirm("Upgrade package", "Upgrade current package?", 0, txt, "Really proceed with the upgrade?");
     free(txt);
     if (c)
       action_upgrade(filename, packagename);
