@@ -92,8 +92,12 @@ void menu_create(void) {
   w_menu[W_MENU_WINDOW] = gtk_menu_item_new_with_mnemonic("_Window");
   w_menu[W_MENU_WINDOW_DEFAULT] = gtk_menu_item_new_with_mnemonic("_Default size");
   w_menu[W_MENU_HELP] = gtk_menu_item_new_with_mnemonic("_Help");
-  w_menu[W_MENU_HELP_CONTENTS] = gtk_image_menu_item_new_from_stock(GTK_STOCK_HELP, acc);
+  
+#if GTK_CHECK_VERSION(2, 6, 0)
   w_menu[W_MENU_HELP_ABOUT] = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, acc);
+#else
+  w_menu[W_MENU_HELP_ABOUT] = gtk_image_menu_item_new_from_stock(GTK_STOCK_HELP, acc);
+#endif
 
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(w_menu[W_MENU_FILE]), menu_file);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu_file), w_menu[W_MENU_FILE_QUIT]);
@@ -122,7 +126,7 @@ void menu_create(void) {
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(w_menu[W_MENU_WINDOW]), menu_window);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu_window), w_menu[W_MENU_WINDOW_DEFAULT]);
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(w_menu[W_MENU_HELP]), menu_help);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu_help), w_menu[W_MENU_HELP_CONTENTS]);
+  /*  gtk_menu_shell_append(GTK_MENU_SHELL(menu_help), w_menu[W_MENU_HELP_CONTENTS]); */
   gtk_menu_shell_append(GTK_MENU_SHELL(menu_help), w_menu[W_MENU_HELP_ABOUT]);
 
   gtk_menu_bar_append(GTK_MENU_BAR(w_menu[W_MENU]), w_menu[W_MENU_FILE]);
